///////////////////////////////////////////////////////////////////////////////
// Projet :  Recherche de solution du Solitaire
// Fichier : Plateau.cpp
// Auteur :  Sylvain Togni
// Date :    08/03/2009
///////////////////////////////////////////////////////////////////////////////
#include "Plateau.h"
#include <istream>
#include <ostream>
#include <string>
#include <stdexcept>
#include <cctype>
#include <algorithm>

///////////////////////////////////////////////////////////////////////////////
// Constructeurs
///////////////////////////////////////////////////////////////////////////////
Plateau::Plateau()
: maHauteur(0)
, maLargeur(0)
{
}

Plateau::Plateau(std::size_t hauteur, std::size_t largeur)
: mesCases(hauteur*largeur, ' ')
, maHauteur(hauteur)
, maLargeur(largeur)
{
}

//////////////////////////////////////////////////////////////////////////////
// Comparaison
///////////////////////////////////////////////////////////////////////////////
bool Plateau::operator==(Plateau const& autre) const
{
    return mesCases == autre.mesCases
        && hauteur() == autre.hauteur()
        && largeur() == autre.largeur();
}

//////////////////////////////////////////////////////////////////////////////
// Ordre
///////////////////////////////////////////////////////////////////////////////
bool Plateau::operator<(Plateau const& autre) const
{
    return mesCases < autre.mesCases;
}

///////////////////////////////////////////////////////////////////////////////
// Inversion pions/cases vides
///////////////////////////////////////////////////////////////////////////////
Plateau Plateau::inverse() const
{
    Plateau resultat(*this);
    for(std::size_t i = 0; i < hauteur(); ++i)
    {
        for(std::size_t j = 0; j < largeur(); ++j)
        {
            switch(resultat(i, j))
            {
            case ' ': resultat(i, j) = ' '; break;
            case '.': resultat(i, j) = 'x'; break;
            case 'x': resultat(i, j) = '.'; break;
            }
        }
    }
    return resultat;
}

///////////////////////////////////////////////////////////////////////////////
// Accs divers
///////////////////////////////////////////////////////////////////////////////
std::size_t Plateau::memoireAllouee() const
{
    return mesCases.capacity();
}

std::size_t Plateau::nombreDeCasesVides() const
{
    return std::count(mesCases.begin(), mesCases.end(), '.');
}

std::size_t Plateau::nombreDePions() const
{
    return std::count(mesCases.begin(), mesCases.end(), 'x');
}

///////////////////////////////////////////////////////////////////////////////
// Lecture format texte
///////////////////////////////////////////////////////////////////////////////
std::istream& operator>>(std::istream& source, Plateau& p)
{
    // 1er tape : lecture des lignes et calcule de la largeur
    std::vector<std::string> lignes;
    std::size_t largeur = 0;
    std::string ligne;
    while(std::getline(source, ligne))
    {
        // Supression des espaces inutiles en fin de ligne
        while(!ligne.empty() && std::isspace(ligne[ligne.size() - 1]))
        {
            ligne.resize(ligne.size() - 1);
        }

        // Vrification des valeurs
        if (ligne.find_first_not_of(" x.") == std::string::npos
            && !ligne.empty())
        {
            // Ajout de la ligne
            lignes.push_back(ligne);
            largeur = std::max(largeur, ligne.size());
        }
    }
    if (!source.eof())
    {
        throw std::logic_error("lecture disque");
    }

    // 2me tape : construction du plateau
    p = Plateau(lignes.size(), largeur);
    for(std::size_t i = 0; i < p.hauteur(); ++i)
    {
        for(std::size_t j = 0; j < p.largeur(); ++j)
        {
            p(i, j) = j < lignes[i].size() ? lignes[i][j] : ' ';
        }
    }
    return source;
}

///////////////////////////////////////////////////////////////////////////////
// criture formate
///////////////////////////////////////////////////////////////////////////////
std::ostream& operator<<(std::ostream& dest, Plateau const& p)
{
    // entte
    dest << "  ";
    for(std::size_t j = 0; j < p.largeur(); ++j)
    {
        dest << static_cast<char>('a' + j);
    }
    dest << '\n';
    dest << "  " << std::string(p.largeur(), '-') << '\n';

    // cases
    for(std::size_t i = 0; i < p.hauteur(); ++i)
    {
        dest << static_cast<char>('1' + i) << '|';
        for(std::size_t j = 0; j < p.largeur(); ++j)
        {
            dest << p(i, j);
        }
        dest << '\n';
    }
    return dest;
}
