///////////////////////////////////////////////////////////////////////////////
// Projet :  Recherche de solution du Solitaire
// Fichier : Plateau.h
// Auteur :  Sylvain Togni
// Date :    08/03/2009
///////////////////////////////////////////////////////////////////////////////
#ifndef PLATEAU_H
#define PLATEAU_H

#include <vector>
#include <iosfwd>

///////////////////////////////////////////////////////////////////////////////
// Classe reprsentant un plateau du jeu du Solitaire.
// Les cases peuvent avoir trois valeurs diffrentes :
//     ' ' : reprsente une case  l'extrieur du plateau
//     '.' : reprsente une case vide (sans pion)
//     'x' : reprsente une case occupe par un pion
///////////////////////////////////////////////////////////////////////////////
class Plateau
{
public:
    // Construit un plateau de taille nulle
                        Plateau();

    // Construit un plateau de taille hauteur*largeur vide (' ')
                        Plateau(std::size_t hauteur, std::size_t largeur);

    // Accs aux dimensions
    std::size_t         hauteur() const {return maHauteur;}
    std::size_t         largeur() const {return maLargeur;}

    // Accs en lecture  la case (i, j)
    char                operator()(std::size_t i, std::size_t j) const
    {
        return mesCases[i*maLargeur + j];
    }

    // Accs en criture  la case (i, j)
    char&               operator()(std::size_t i, std::size_t j)
    {
        return mesCases[i*maLargeur + j];
    }

    // Fonctions de comparaison et d'ordre
    bool                operator==(Plateau const& autre) const;
    bool                operator<(Plateau const& autre) const;

    // Fonction d'inversion pions/cases vides
    Plateau             inverse() const;

    // Accs divers
    std::size_t         nombreDeCasesVides() const;
    std::size_t         nombreDePions() const;
    std::size_t         memoireAllouee() const;

private:
    std::vector<char>   mesCases;
    std::size_t         maHauteur;
    std::size_t         maLargeur;
};

// Fonctions de formatage/dformatage textuelles
std::istream& operator>>(std::istream& source, Plateau& p);
std::ostream& operator<<(std::ostream& dest, Plateau const& p);

#endif // PLATEAU_H
