///////////////////////////////////////////////////////////////////////////////
// Projet :  Recherche de solution du Solitaire
// Fichier : Solveur.h
// Auteur :  Sylvain Togni
// Date :    08/03/2009
///////////////////////////////////////////////////////////////////////////////
#ifndef SOLVEUR_H
#define SOLVEUR_H

#include <string>

class Plateau;

///////////////////////////////////////////////////////////////////////////////
// Cette classe contient l'algorithme de recherche de chemin entre deux
// plateaux. Pour plus de dtails, reportez vous au fichier README.
///////////////////////////////////////////////////////////////////////////////
class Solveur
{
public:
                        Solveur();

    // Recherche d'une solution (i.e. une liste de coups valides) qui mne du
    // plateau p1 au plateau p2. Retourne false si aucune solution existe.
    bool                rechercheChemin(Plateau const& p1, Plateau const& p2);

    // Accs au chemin trouv (liste des coups).
    std::string const&  solution() const {return maSolution;}

    // Nombre de position explores pendant la recherche.
    std::size_t         nombreDePositions() const {return monNombreDePositions;}

    // Mmoire maximale consomme pendant la recherche (approximatif).
    std::size_t         memoireUtilisee() const {return maMemoireUtilisee;}

private:
    std::string         maSolution;
    std::size_t         monNombreDePositions;
    std::size_t         maMemoireUtilisee;

    bool                beamSearch(Plateau const& p0, Plateau const& p1,
                                   std::size_t taille);
};

///////////////////////////////////////////////////////////////////////////////
#endif // SOLVEUR_H
